package Network.Algorithm;


import Network.Algorithm.FlowAugmentingDecreasingPathsAlgorithm.*;
import Network.Algorithm.SimplexAlgorithm.*;
import Network.Algorithm.TreeAlgorithm.*;



public class NodeAlgInfoFactory
{
    
    private NodeAlgInfoStack flow;    //FlowAugmentingDecreasingPathsAlgNodeInfo only
    private NodeAlgInfoStack simplex; //SimplexAlgNodeInfo only
    private NodeAlgInfoStack tree;    //TreeNodeInfo only
    
    
    public NodeAlgInfoFactory(int initialSize_, int capacityIncrement_)
    {
        flow = new NodeAlgInfoStack(initialSize_, capacityIncrement_);
        simplex = new NodeAlgInfoStack(initialSize_, capacityIncrement_);
        tree = new NodeAlgInfoStack(initialSize_, capacityIncrement_);
    }
    
    
    public final FlowAugmentingDecreasingPathsAlgNodeInfo makeFlowInfo()
    {
        FlowAugmentingDecreasingPathsAlgNodeInfo info =
            (FlowAugmentingDecreasingPathsAlgNodeInfo)flow.pop();
        if (info != null)
            return info;
        else
            return new FlowAugmentingDecreasingPathsAlgNodeInfo();
    }
    
    public final SimplexAlgNodeInfo makeSimplexInfo()
    {
        SimplexAlgNodeInfo info =
            (SimplexAlgNodeInfo)simplex.pop();
        if (info != null)
            return info;
        else
            return new SimplexAlgNodeInfo();
    }
    
    public final TreeNodeInfo makeTreeInfo()
    {
        TreeNodeInfo info =
            (TreeNodeInfo)tree.pop();
        if (info != null)
            return info;
        else
            return new TreeNodeInfo();
    }
    
    public void collectInfo(NodeAlgInfo info)
    {
        if (info instanceof TreeNodeInfo)
            tree.push(info);
        else if (info instanceof SimplexAlgNodeInfo)
            simplex.push(info);
        else if (info instanceof FlowAugmentingDecreasingPathsAlgNodeInfo)
            flow.push(info);
    }
    
}


final class NodeAlgInfoStack
{
    
    private NodeAlgInfo[] stack;    //null not a valid entry
    private int size = 0;          //Number of valid entries on stack
    private int capacityIncrement; //Amount by which to increase stack length when we resize


    NodeAlgInfoStack(int initialSize_, int capacityIncrement_)
    {
        stack = new NodeAlgInfo[initialSize_];
        if (capacityIncrement_ < 1)
            throw new IllegalArgumentException("NodeAlgInfoStack: capacityIncrement_ must be >= 1.");
        else
            capacityIncrement = capacityIncrement_;
    }
    
    
    synchronized void push(NodeAlgInfo info)
    {
        if (size == stack.length)
            resize();
        stack[size++] = info;
    }
    
    //Returns null if empty
    synchronized NodeAlgInfo pop()
    {
        if (size > 0)
            return stack[--size];
        else
            return null;
    }
    
    int getSize()
    {
        return (size);
    }
    
    synchronized private void resize()
    {
        NodeAlgInfo[] resized = new NodeAlgInfo[size + capacityIncrement];
        System.arraycopy(stack, 0, resized, 0, size);
        stack = resized;
    }
    
}

