package Network.Algorithm.SimplexAlgorithm;

import Network.Representation.*;



final class ArrayCandidateList implements CandidateList {
    
    private Arc[] arcList; //Contains list.  Only first listSize entries valid.
                           //arcList.length is maxListSize.
    private int listSize;  //Number of valid Arcs in list array, *not* the
                           //same as arcList.length
    
    private SimplexAlgorithm ownerAlgorithm;
    private NodeList lastNodeTestedInList;
    private int maxNoOfGivenArcs;
    private int noOfGivenArcs;
    
    
    public ArrayCandidateList(int size, int cycleLength, Network network,
                              SimplexAlgorithm owner)
    {
        initialize(size, cycleLength, network, owner);
    }
    
    
    public final void initialize(int size, int cycleLength, Network network,
                                 SimplexAlgorithm owner)
    {
        cleanList();
        arcList = new Arc[size];
        ownerAlgorithm = owner;
        lastNodeTestedInList = network.getNodeListEnd();
        maxNoOfGivenArcs = cycleLength;
        noOfGivenArcs = cycleLength;
    }
    
    
    public final void cleanList()
    {
        arcList = null;
        listSize = 0;
    }
    
    
    public final Arc bestCandidate(double optimalityTolerance, boolean isDoubleCosts) {
        Arc bestArcSoFar;                       /*bestarc*/
        double bestCostSoFar;                   /*bestcost*/
        Arc bestArcForCurrentNode;              /*bestnodearc*/
        double bestCostForCurrentNode;          /*bestnodecost*/
        Arc dummyArcCursor;                     /*tmparcp*/
        Arc currentArc;                         /*tmparc*/
        ArcList currentArcIterator; //replaces currentArc, tmparc*/
        double checkedRC;              /*checkcost*/
        Node nodeToPrice;                       /*nodetoprice*/
        NodeList nodeToPriceIterator;
        Node nodeCursor;                        /*whichnode*/
        NodeList nodeCursorIterator;
        Node stoppingNode;                      /*stopnode*/
        NodeList stoppingNodeIterator;
        double reducedCost;                     /*dualval*/
        
        bestArcSoFar=null;
        bestCostSoFar=0;
        if (noOfGivenArcs < maxNoOfGivenArcs){
            for (int currentListIndex=0; currentListIndex<listSize; currentListIndex++) {
                currentArc=arcList[currentListIndex];
                if (ownerAlgorithm.getAlgArcInfo(currentArc).flow==0){
                    checkedRC=  ownerAlgorithm.getAlgNodeInfo(currentArc.getTailNode()).dualVariable+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                }
                else if (ownerAlgorithm.getAlgArcInfo(currentArc).flow==currentArc.getUpperBound()){
                    checkedRC= -ownerAlgorithm.getAlgNodeInfo(currentArc.getTailNode()).dualVariable-currentArc.getCost()+ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                }
                else{
                    checkedRC=  ownerAlgorithm.getAlgNodeInfo(currentArc.getTailNode()).dualVariable+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                    if (checkedRC>0) checkedRC=-checkedRC;
                }
                if (checkedRC<bestCostSoFar) {
                    bestCostSoFar=checkedRC;
                    bestArcSoFar=currentArc;
                }
            }
        }
        
        if ((isDoubleCosts) && (bestArcSoFar!=null) && (bestCostSoFar > (-optimalityTolerance * (1.0+Math.abs(bestArcSoFar.getCost())+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getTailNode()).dualVariable)+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getHeadNode()).dualVariable))))) {
            bestArcSoFar=null;
        }

                            
        if (bestArcSoFar==null){
            listSize = 0;
            stoppingNodeIterator=lastNodeTestedInList;
            nodeCursorIterator=stoppingNodeIterator;
            do {
                nodeCursorIterator=ownerAlgorithm.getTargetNetwork().getNextNodeInList(nodeCursorIterator);
                nodeToPriceIterator=nodeCursorIterator;
                bestCostForCurrentNode=0;
                bestArcForCurrentNode=null;
                currentArcIterator=nodeToPriceIterator.elem.getOutBoundArcs();
                reducedCost=ownerAlgorithm.getAlgNodeInfo( (nodeToPriceIterator.elem) ).dualVariable;
                while (currentArcIterator!=null) {
                    currentArc=currentArcIterator.elem;
                    if ((ownerAlgorithm.getAlgArcInfo(currentArc).flow)==0) {
                        checkedRC= reducedCost+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                    }
                    else if ((ownerAlgorithm.getAlgArcInfo(currentArc).flow)==currentArc.getUpperBound()) {
                        checkedRC=-reducedCost-currentArc.getCost()+ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                    }
                    else {
                        checkedRC= reducedCost+currentArc.getCost()-ownerAlgorithm.getAlgNodeInfo(currentArc.getHeadNode()).dualVariable;
                        if (checkedRC>0) {checkedRC=-checkedRC;}
                    }
                    if (checkedRC<bestCostForCurrentNode){
                        bestCostForCurrentNode=checkedRC;
                        bestArcForCurrentNode=currentArc;
                    }
                    currentArcIterator=currentArcIterator.next;
                }
                if (bestArcForCurrentNode!=null) {
                    /**************************************************************************/
                    arcList[listSize++] = bestArcForCurrentNode;
                    if (bestCostForCurrentNode<bestCostSoFar) {
                        bestCostSoFar=bestCostForCurrentNode;
                        bestArcSoFar=bestArcForCurrentNode;
                    }
                }
            }
            while ((listSize<arcList.length) && (!(nodeCursorIterator.equals(stoppingNodeIterator))));
            
            if ((isDoubleCosts==true) && (bestArcSoFar!=null) && (bestCostSoFar > (-optimalityTolerance * (1.0+Math.abs(bestArcSoFar.getCost())+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getTailNode()).dualVariable)+Math.abs(ownerAlgorithm.getAlgNodeInfo(bestArcSoFar.getHeadNode()).dualVariable))))) {
                bestArcSoFar=null;
            }
            
            
            
            lastNodeTestedInList=nodeCursorIterator;
            noOfGivenArcs=0;
        }
        noOfGivenArcs++;
        return(bestArcSoFar);
    }
}
