package Network.Representation;

import Network.Algorithm.*;
import Network.Representation.*;

import java.util.*;

/**
 * A default implementation for the interface <code>Arc</code>.
 */
public class ArcImplementation implements Arc {
    
    public static int defaultInfoSize = 10;
    
    Node tailNode;
    Node headNode;
    double upperBound;
    double cost;
    Vector arcAlgorithmInfo;

    /**
     * Creates an instance of ArcImplementation with given cost and upper bound.
     *
     * @param   arcCost cost of the arc to be created
     * @param   arcUpperBound   upper bound for the arc to be created
     */
    public ArcImplementation (double arcCost,double arcUpperBound){
        cost=arcCost;
        upperBound=arcUpperBound;
        tailNode=null;
        headNode=null;
        arcAlgorithmInfo=new Vector(defaultInfoSize);
    }
    
    /**
     * Creates and instance of ArcImplementation with given cost and upper bound and adds this arc to the given
     * network between given head and tail nodes.
     *
     * @param   targetNetwork   network that the arc is going to be added
     * @param   fromNode    tail node of the arc
     * @param   toNode  head node of the arc 
     * @param   arcCost cost of the arc to be created
     * @param   arcUpperBound   upper bound for the arc to be created
     */
    public ArcImplementation (Network targetNetwork, Node fromNode, Node toNode, double arcCost, double arcUpperBound){
        cost=arcCost;
        upperBound=arcUpperBound;
        arcAlgorithmInfo=new Vector(defaultInfoSize);
        targetNetwork.addArc(this, fromNode, toNode);
    }
        
     
        
    public void initializeArc(double arcCost,double arcUpperBound){
      if ( (tailNode==null) && (headNode==null) ) {
            cost=arcCost;
            upperBound=arcUpperBound;
            arcAlgorithmInfo.setSize(0);
        }
        else {
            throw new ArcError("can't initialize arc: arc belongs to a network");
        }
    
    }
    
    
   
    public final Node getTailNode(){
        return tailNode;
    }

    /**
     * This procedure should normally NOT be used by users.
     */
    public void setTailNode(Node setNode){
        tailNode=setNode;          
    }
    
    
    public final Node getHeadNode(){
        return headNode;
    }
    
         
    /**
     * This procedure should normally NOT be used by users.
     */
    public void setHeadNode(Node setNode){
        headNode=setNode;
    }
                
    public final Vector getArcAlgorithmInfo(){
        return arcAlgorithmInfo;
    }
    
    public final void cleanArcAlgorithmInfo(){
        for (int i = arcAlgorithmInfo.size(); i-- > 0; )
            AlgorithmImplementation.arcInfoFactory.collectInfo((ArcAlgInfo)arcAlgorithmInfo.elementAt(i));
        arcAlgorithmInfo.setSize(0);
    }
       
    public final double getCost(){
        return cost;
    }
    
    /**
     * This procedure should normally NOT be used by users.
     */
    public final void setCost(double setCost){
        cost = setCost;
    }
    
    public final double getUpperBound(){
        return upperBound;
    }
    
    public void setUpperBound( double ub )
    {
        upperBound = ub;
    }
    
    /**
     * Return the string representation of this object with cost, upperboud, tailNode.toString() and headNode.toString()
     * seperated by spaces.
     */
     
    public String toString() {
        return Double.toString(cost)+" "+Double.toString(upperBound)+" "+
               tailNode.getClass().getName()+" : "+tailNode.toString()+" -> "+
               headNode.getClass().getName()+" : "+headNode.toString();
    }
   
}
