package Network.Representation;


public final class ArcList {
    
    public Arc elem;
    
    public ArcList next;
    
    
    public ArcList(Arc elem_, ArcList next_)
    {
        elem = elem_;
        next = next_;
    }
    
    public ArcList(Arc elem_)
    {
        elem = elem_;
        next = null;
    }
    
    
    public final void initialize(Arc elem_, ArcList next_)
    {
        elem = elem_;
        next = next_;
    }
    
    public final void initialize(Arc elem_)
    {
        elem = elem_;
        next = null;
    }
    
    
    public final void addAfter(ArcList arcList)
    {
        ArcList temp = next;
        next = arcList;
        arcList.next = temp;
    }
    
    public final ArcList find(Arc elem_)
    {
        ArcList nextArcList = this;
        while (nextArcList != null) {
            if (nextArcList.elem.equals(elem_))
                return (nextArcList);
            nextArcList = nextArcList.next;
        }
        return (null);
    }
    
    public final boolean contains(Arc elem_)
    {
        return (find(elem_) != null);
    }
    
    public final ArcList remove(Arc elem_)
    {
        ArcList lastArcList = this;
        ArcList currentArcList = this;
        while (currentArcList != null) {
            ArcList nextArcList = currentArcList.next;
            if (currentArcList.elem.equals(elem_)) {
                lastArcList.next = currentArcList.next;
                return (currentArcList);
            }
            lastArcList = currentArcList;
            currentArcList = nextArcList;
        }
        return (null);
    }
    
}


final class ArcListStack {

    private ArcList[] stack;
    private int size = 0;
    private int capacity;
    private int capacityIncrement;
    
    
    ArcListStack(int capacity_, int capacityIncrement_)
    {
        capacity = capacity_;
        capacityIncrement = capacityIncrement_;
        stack = new ArcList[capacity];
    }
    
    
    final synchronized void push(ArcList arcList)
    {
        if (size >= capacity)
            resize();
        stack[size++] = arcList;
    }
    
    final synchronized ArcList pop()
    {
        if (size > 0)
            return (stack[--size]);
        else
            return (null);
    }
    
    final synchronized private void resize()
    {
        ArcList[] newStack = new ArcList[capacity + capacityIncrement];
        System.arraycopy(stack, 0, newStack, 0, stack.length - 1);
        stack = newStack;
        capacity += capacityIncrement;
    }

}
